;--------------------------------------------------------------------------
; crt0.s - Generic crt0.s for a Z80
;
; Written by Christian Julius in 10.2014 for a Minimum Z80 System
; with Mostek 3801 Multi I/O Chip
;--------------------------------------------------------------------------

	.module crt0			; Modul Name fr den Linker
 
	; Memory Map Adressen der Zeropage (vom Benutzer definierbar)
	; 0x00 					Reset Vector und Aufruf Hauptprogramm
	; 0x38					Mode 1 Int Vektor
	; 0x40					Mode 2 Vektor Tabelle fr STI Mostek Multi I/O
	; 0x66					NMI Reset Vektor
	; 0x80					Copyright String
	; 0x100					User Programm

	; Definitions
	stack  				= 0xffff	; fix
	adr_vec_table		= 0x40		; frei
	adr_copyright       = 0x80		; frei
	adr_nmi				= 0x66		; fix

	; Globale Funktionen und Variablen fr das C-Programm

    .globl	_main
	.globl  _nmi_vint 		; Funktion des NMI Handlers

	; Interruptroutinen fr den Mostek MK3801 Multi I/O Baustein
	; Manual STI Baustein MK3801 Figure 7
	.globl  _int_sti_gpi_0	; General Purpose Interrupt 0
	.globl  _int_sti_gpi_1	; General Purpose Interrupt 1
	.globl  _int_sti_gpi_2 	; General Purpose Interrupt 2
	.globl  _int_sti_gpi_3	; General Purpose Interrupt 3
	.globl  _int_sti_timer_d
	.globl  _int_sti_timer_c
	.globl  _int_sti_gpi_4	; General Purpose Interrupt 4
	.globl  _int_sti_gpi_5	; General Purpose Interrupt 5
	.globl  _int_sti_timer_b
	.globl  _int_sti_transmit_error
	.globl  _int_sti_transmit_buffer_empty
	.globl  _int_sti_receive_error
	.globl  _int_sti_receive_buffer_full
	.globl  _int_sti_timer_a
	.globl  _int_sti_gpi_6	; General Purpose Interrupt 6
	.globl  _int_sti_gpi_7	; General Purpose Interrupt 7
	; Mode 1, alles auf eine Routine umleiten
	.globl  _int_sti_all
	
	.globl l__INITIALIZER
    .globl s__INITIALIZER
    .globl s__INITIALIZED

	; -------------------------------------------------------------

	.area	_HEADER (ABS)	
	; Reset vector bei Kaltstart und Sprung ins User Programm
	.org 	0
	 jp init
	
	;///////////////////////////////////////////////////////////
	;// Mode 1 Interrupt
	.org 0x38
	jp _int_sti_all


	;///////////////////////////////////////////////////////////
	; Tabelle der Mode 2 Int Vektoren auf die Handler Funktionen im C Modul
	; Reihenfolge beachten !!! Siehe Manual Mostek 3801, Interrupt Tabelle
	.org 0x40
	.dw (_int_sti_gpi_0)
	.dw (_int_sti_gpi_1)
	.dw (_int_sti_gpi_2)
	.dw (_int_sti_gpi_3)
	.dw (_int_sti_timer_d)
	.dw (_int_sti_timer_c)
	.dw (_int_sti_gpi_4)
	.dw (_int_sti_gpi_5)
	.dw	(_int_sti_timer_b)
	.dw	(_int_sti_transmit_error)
	.dw	(_int_sti_transmit_buffer_empty)
	.dw	(_int_sti_receive_error)
	.dw	(_int_sti_receive_buffer_full)
	.dw	(_int_sti_timer_a)
	.dw	(_int_sti_gpi_6)
	.dw	(_int_sti_gpi_7)

	; NMI Interrupt 0x66, muss mit retn abgeschlossen werden
	.org  0x66
	jp _nmi_vint		; Aufruf Handler im C Modul main.c

	; Ordering of segments for the linker - copied from sdcc crt0.s
	  .area  _CODE
	  .area  _INITIALIZER
	  .area  _HOME
	  .area  _GSINIT
	  .area  _GSFINAL
	  .area  _DATA
	  .area  _INITIALIZED
	  .area  _BSEG
	  .area  _BSS
	  .area  _HEAP

	  .area  _CODE
	
	;; ------- Start des Hauptprogramms nach der Zeropage  -----------
	
	init:
	
	ld	sp,#stack	;; Stack an Spitze des RAM legen
	
	; Ram ausnullen
	xor     a             ; clear a and carry
	ld      bc,#0xdfff    ; ram size left
	ld      hl,#0x2000    ; starting from 2000
	ld      de,#0x2001
	ld      (hl),a		  ; 0 -> (2000)	
	ldir                  ; (HL) -> (DE)
	
	
	; Explicitly initialized global variables.
	ld  bc, #l__INITIALIZER
	ld  a, b
	or  a, c
	jr  Z, weiter
	ld  de, #s__INITIALIZED
	ld  hl, #s__INITIALIZER
	ldir
	
weiter:	

	jp 	_main		; ###### Hauptprogramm aufrufen #######
endlos:
	jr endlos		; Sicher ist sicher

	.area _HOME


 
	
	
	
	
